@@ -40,9 +40,12 @@
 #define HOME_DIR g_get_home_dir()
 
 #define GPRS_GCONF_PATH "/system/osso/connectivity/network_type/GPRS"
-#define GPRS_GCONF_RX GPRS_GCONF_PATH  "/gprs_rx_bytes"
-#define GPRS_GCONF_TX GPRS_GCONF_PATH  "/gprs_tx_bytes"
-#define GPRS_GCONF_RESET GPRS_GCONF_PATH  "/gprs_reset_time"
+#define GPRS_GCONF_HOME_RX GPRS_GCONF_PATH  "/gprs_home_rx_bytes"
+#define GPRS_GCONF_HOME_TX GPRS_GCONF_PATH  "/gprs_home_tx_bytes"
+#define GPRS_GCONF_HOME_RESET GPRS_GCONF_PATH  "/gprs_home_reset_time"
+#define GPRS_GCONF_ROAMING_RX GPRS_GCONF_PATH "/gprs_roaming_rx_bytes"
+#define GPRS_GCONF_ROAMING_TX GPRS_GCONF_PATH "/gprs_roaming_tx_bytes"
+#define GPRS_GCONF_ROAMING_RESET GPRS_GCONF_PATH "/gprs_roaming_reset_time"
 
 #define _tran(String) dgettext("hildon-libs", String)
 #define _(String) gettext (String)
@@ -83,8 +86,10 @@ struct _PersonalGprsMonPrivate
 	gboolean showResetDate;
 	gboolean compactMode;
 	gdouble version;
+	gboolean combineTotals;
 	
 	gboolean isPressed;
+	gboolean showHome;
 	
 	guint eventID;
 };
@@ -113,37 +118,44 @@ void personal_gprs_mon_read_settings ( PersonalGprsMon *self )
             g_error_free (error);
             error = NULL;
         }
-		self->priv->binaryPrefix = g_key_file_get_boolean (keyFile, "config", "binaryPrefix", &error);
-		if (error) {
-			self->priv->binaryPrefix = FALSE;
-			g_error_free (error);
-			error = NULL;
-		}
-		self->priv->showResetDate = g_key_file_get_boolean (keyFile, "config", "showResetDate", &error);
-		if (error) {
-			self->priv->showResetDate = FALSE;
-			g_error_free (error);
-			error = NULL;
-		}
-		self->priv->compactMode = g_key_file_get_boolean (keyFile, "config", "compactMode", &error);
-		if (error) {
-			self->priv->compactMode = FALSE;
-			g_error_free (error);
-			error = NULL;
-		}
-		// Introduced in version 0.4-1 to allow change in refresh intervals
-		self->priv->version = g_key_file_get_double (keyFile, "config", "version", &error);
-		if (error) {
-			self->priv->version = 0.3;
-			g_error_free (error);
-			error = NULL;
-		}
+	self->priv->binaryPrefix = g_key_file_get_boolean (keyFile, "config", "binaryPrefix", &error);
+	if (error) {
+		self->priv->binaryPrefix = FALSE;
+		g_error_free (error);
+		error = NULL;
+	}
+	self->priv->showResetDate = g_key_file_get_boolean (keyFile, "config", "showResetDate", &error);
+	if (error) {
+		self->priv->showResetDate = FALSE;
+		g_error_free (error);
+		error = NULL;
+	}
+	self->priv->compactMode = g_key_file_get_boolean (keyFile, "config", "compactMode", &error);
+	if (error) {
+		self->priv->compactMode = FALSE;
+		g_error_free (error);
+		error = NULL;
+	}
+	// Introduced in version 0.4-1 to allow change in refresh intervals
+	self->priv->version = g_key_file_get_double (keyFile, "config", "version", &error);
+	if (error) {
+		self->priv->version = 0.3;
+		g_error_free (error);
+		error = NULL;
+	}
+	self->priv->combineTotals = g_key_file_get_boolean (keyFile, "config", "combineTotals", &error);
+	if (error) {
+		self->priv->combineTotals = TRUE;
+		g_error_free (error);
+		error = NULL;
+	}
     } else {
         self->priv->refresh = 1;
 		self->priv->binaryPrefix = FALSE;
 		self->priv->showResetDate = FALSE;
 		self->priv->compactMode = FALSE;
 		self->priv->version = 0.3;
+		self->priv->combineTotals = TRUE;
     }
 	
 	if (self->priv->version < 0.4) {
@@ -197,6 +209,7 @@ void personal_gprs_mon_write_settings (PersonalGprsMon *self)
 	g_key_file_set_boolean (keyFile, "config", "showResetDate", self->priv->showResetDate);
 	g_key_file_set_boolean (keyFile, "config", "compactMode", self->priv->compactMode);
 	g_key_file_set_double (keyFile, "config", "version", 0.4);
+	g_key_file_set_boolean (keyFile, "config", "combineTotals", self->priv->combineTotals);
 	
 	filename = g_strconcat (HOME_DIR, "/.personal_gprs_mon", NULL);
 	fileData = g_key_file_to_data (keyFile, &size, NULL);
@@ -219,6 +232,9 @@ void personal_gprs_mon_button_release (GtkWidget *widget, GdkEventButton *event,
 {
 	//g_warning ("personal_gprs_mon_button_release");
 	self->priv->isPressed = FALSE;
+	if (!self->priv->combineTotals) {
+		self->priv->showHome = !self->priv->showHome;
+	}
 	personal_gprs_mon_update_content (self);
 	gtk_widget_queue_draw (GTK_WIDGET (self));
 }
@@ -362,48 +378,105 @@ void personal_gprs_mon_content_create (PersonalGprsMon *self)
 gboolean personal_gprs_mon_update_content (PersonalGprsMon *self)
 {
 	//g_warning ("personal_gprs_mon_update_content");
-	gchar *uploadString, *downloadString;
-	gchar *resetString;
-	gint upload, download, resetTime;
+	gchar *homeUploadString, *homeDownloadString;
+	gchar *homeResetString;
+	gchar *roamingUploadString, *roamingDownloadString;
+	gchar *roamingResetString;
+	guint homeUpload, homeDownload, homeResetTime;
+	guint roamingUpload, roamingDownload, roamingResetTime;
 	GError *error = NULL;
 
-	uploadString = gconf_client_get_string (self->priv->client, GPRS_GCONF_TX, &error);
+	homeUploadString = gconf_client_get_string (self->priv->client, GPRS_GCONF_HOME_TX, &error);
 	if (error) {
 		g_warning
-			("Error trying to get gconf variable %s", GPRS_GCONF_TX);
-		uploadString = "0";
+			("Error trying to get gconf variable %s", GPRS_GCONF_HOME_TX);
+		homeUploadString = "0";
 		g_error_free (error);
 		error = NULL;
 	}
-	downloadString = gconf_client_get_string (self->priv->client, GPRS_GCONF_RX, &error);
+	homeDownloadString = gconf_client_get_string (self->priv->client, GPRS_GCONF_HOME_RX, &error);
 	if (error) {
 		g_warning
-			("Error trying to get gconf variable %s", GPRS_GCONF_RX);
-		downloadString = "0";
+			("Error trying to get gconf variable %s", GPRS_GCONF_HOME_RX);
+		homeDownloadString = "0";
 		g_error_free (error);
 		error = NULL;
 	}
-	resetString = gconf_client_get_string (self->priv->client, GPRS_GCONF_RESET, &error);
+	homeResetString = gconf_client_get_string (self->priv->client, GPRS_GCONF_HOME_RESET, &error);
 	if (error) {
 		g_warning
-			("Error trying to get gconf variable %s", GPRS_GCONF_RESET);
-		resetString = "0";
+			("Error trying to get gconf variable %s", GPRS_GCONF_HOME_RESET);
+		homeResetString = "0";
+		g_error_free (error);
+		error = NULL;
+	}
+	roamingUploadString = gconf_client_get_string (self->priv->client, GPRS_GCONF_ROAMING_TX, &error);
+	if (error) {
+		g_warning
+			("Error trying to get gconf variable %s", GPRS_GCONF_ROAMING_TX);
+		roamingUploadString = "0";
+		g_error_free (error);
+		error = NULL;
+	}
+	roamingDownloadString = gconf_client_get_string (self->priv->client, GPRS_GCONF_ROAMING_RX, &error);
+	if (error) {
+		g_warning
+			("Error trying to get gconf variable %s", GPRS_GCONF_ROAMING_RX);
+		roamingDownloadString = "0";
+		g_error_free (error);
+		error = NULL;
+	}
+	roamingResetString = gconf_client_get_string (self->priv->client, GPRS_GCONF_ROAMING_RESET, &error);
+	if (error) {
+		g_warning
+			("Error trying to get gconf variable %s", GPRS_GCONF_ROAMING_RESET);
+		roamingResetString = "0";
 		g_error_free (error);
 		error = NULL;
 	}
 	
-	upload = g_ascii_strtoll (uploadString, NULL, 10);
-	download = g_ascii_strtoll (downloadString, NULL, 10);
-	if (resetString != NULL) {
-		resetTime = g_ascii_strtoll (resetString, NULL, 10);
+	homeUpload = g_ascii_strtoull (homeUploadString, NULL, 10);
+	homeDownload = g_ascii_strtoull (homeDownloadString, NULL, 10);
+	if (homeResetString != NULL) {
+		homeResetTime = g_ascii_strtoull (homeResetString, NULL, 10);
 	} else {
-		resetTime = 0;
+		homeResetTime = 0;
+	}
+	roamingUpload = g_ascii_strtoull (roamingUploadString, NULL, 10);
+	roamingDownload = g_ascii_strtoull (roamingDownloadString, NULL, 10);
+	if (roamingResetString != NULL) {
+		roamingResetTime = g_ascii_strtoull (roamingResetString, NULL, 10);
+	} else {
+		roamingResetTime = 0;
 	}
 	
 	gdouble uploadSize;
 	gdouble downloadSize;
 	gchar *uploadUnits, *downloadUnits;
 	
+	guint upload;
+	guint download;
+	guint resetTime;
+
+	if (self->priv->combineTotals) {
+		upload = homeUpload + roamingUpload;
+		download = homeDownload + roamingDownload;
+		if (homeResetTime > roamingResetTime) {
+			resetTime = roamingResetTime;
+		} else {
+			resetTime = homeResetTime;
+		}
+	} else {
+		if (self->priv->showHome) {
+			upload = homeUpload;
+			download = homeDownload;
+			resetTime = homeResetTime;
+		} else {
+			upload = roamingUpload;
+			download = roamingDownload;
+			resetTime = roamingResetTime;
+		}
+	}
 	if (self->priv->binaryPrefix) {
 		if (upload > 1024 && upload <= 1048576) {
 			uploadSize = upload / 1024.0;
@@ -475,10 +548,10 @@ gboolean personal_gprs_mon_update_content (PersonalGprsMon *self)
 		gtk_label_set_text (GTK_LABEL (self->priv->resetContent), dateString);
 	}
 	
-	gint uploadInt = (gint) uploadSize;
-	gint downloadInt = (gint) downloadSize;
-	gint uploadRemainder = (gint) ((uploadSize - uploadInt) * pow (10, 3));
-	gint downloadRemainder = (gint) ((downloadSize - downloadInt) * pow (10, 3));
+	guint uploadInt = (guint) uploadSize;
+	guint downloadInt = (guint) downloadSize;
+	guint uploadRemainder = (guint) ((uploadSize - uploadInt) * pow (10, 3));
+	guint downloadRemainder = (guint) ((downloadSize - downloadInt) * pow (10, 3));
 	gtk_label_set_text (GTK_LABEL (self->priv->uploadContent), g_strdup_printf ("%d.%03d %s", uploadInt, uploadRemainder, uploadUnits));
 	gtk_label_set_markup (GTK_LABEL (self->priv->uploadContent), g_strdup_printf ("%d<small><sup>.%03d</sup></small> %s", uploadInt, uploadRemainder, uploadUnits));
 	gtk_label_set_text (GTK_LABEL (self->priv->downloadContent), g_strdup_printf ("%d.%03d %s", downloadInt, downloadRemainder, downloadUnits));
@@ -546,6 +619,11 @@ void personal_gprs_mon_settings (HDHomePluginItem *hitem, PersonalGprsMon *self)
 	gtk_container_add (GTK_CONTAINER (content_area), compactMode);
 	hildon_check_button_set_active (HILDON_CHECK_BUTTON (compactMode), self->priv->compactMode);
 	
+	GtkWidget *combineTotals = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	gtk_button_set_label (GTK_BUTTON (combineTotals), _("Combine home and roaming totals"));
+	gtk_container_add (GTK_CONTAINER (content_area), combineTotals);
+	hildon_check_button_set_active (HILDON_CHECK_BUTTON (combineTotals), self->priv->combineTotals);
+	
 	gtk_widget_show_all (dialog);
 	
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
@@ -553,6 +631,7 @@ void personal_gprs_mon_settings (HDHomePluginItem *hitem, PersonalGprsMon *self)
 		self->priv->binaryPrefix = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (binaryPrefix));
 		self->priv->showResetDate = hildon_check_button_get_active (HILDON_CHECK_BUTTON (showResetDate));
 		self->priv->compactMode = hildon_check_button_get_active (HILDON_CHECK_BUTTON (compactMode));
+		self->priv->combineTotals = hildon_check_button_get_active (HILDON_CHECK_BUTTON (combineTotals));
 		
 		switch (self->priv->refresh) {
 		case 0:
@@ -781,6 +860,7 @@ static void personal_gprs_mon_init (PersonalGprsMon *self)
 	self->priv = PERSONAL_GPRS_MON_GET_PRIVATE (self);
 	self->priv->eventID = 0;
 	self->priv->isPressed = FALSE;
+	self->priv->showHome = TRUE;
 	
 	hd_home_plugin_item_set_settings (&self->parent, TRUE);
 	g_signal_connect (&self->parent, "show-settings", G_CALLBACK (personal_gprs_mon_settings), self);
